var request = require('supertest')
  , path = require('path')
  , chai = require('chai')
  , should = chai.should()
  , expect = chai.expect()
  , app = require('../../app').app
  , http = require('http')
  , newCasaId
  , port = 3000
  , server
  , request = request('http://localhost:'+port);

var createJSON = {
  name: '',
  email: '',  
}

var updateJSON = {
  name: '',
  email: '',  
}

function startServer(done) {
  server = http.createServer(app).listen(port);

  server.on("listening", function() {
    done();
  });
}

function stopServer(done) {
  server.close();
    done();
}


describe('Casas', function(){
  beforeEach(startServer);
  afterEach(stopServer);

  it('GET Should return all users when calling /casas.json', function(done){
    request
      .get('/casas.json')
      .set('Accept', 'application/json')
      .expect('Content-Type', /json/)
      .expect(200, done);
  });

  it('POST Should create a new Casa', function(done){
    request
      .post('/casas.json')
      .send(createJSON)
      .expect(201)
      .end(function(err, res){
        if (err) throw err;
        var obj = res.body;
        newCasaId = obj._id;

        obj.should.have.property('_id', newCasaId);
        obj.should.have.property('name', createJSON.name);
        obj.should.have.property('email', createJSON.email);

        done();
      });
  });

  it('GET Should return a single Casa when calling /casas/{ID}.json', function(done){
    request
      .get('/casas/'+newCasaId+'.json')
      .set('Accept', 'application/json')
      .expect('Content-Type', /json/)
      .expect(200)
      .end(function(err, res){
        if (err) throw err;
        var obj = res.body;

        obj.should.have.property('_id', newCasaId);
        obj.should.have.property('name', createJSON.name);
        obj.should.have.property('email', createJSON.email);

        done();
      });
  });

  it('GET Should return a 404 when calling /casas/{ID}.json with an ID that doesn\'t exist', function(done){
    request
      .get('/casas/xxxxxxxxxxxxxxxx.json')
      .set('Accept', 'application/json')
      .expect('Content-Type', /json/)
      .expect(500, done);
  });

  it('PUT Should update an existing Casa when calling /casas/{ID}.json', function(done){
    request
      .put('/casas/'+newCasaId+'.json')
      .set('Accept', 'application/json')
      .expect('Content-Type', /json/)
      .send(updateJSON)
      .expect(200)
      .end(function(err, res){
        if (err) throw err;
        var obj = res.body;

        obj.should.have.property('_id', newCasaId);
        obj.should.have.property('name', updateJSON.name);
        obj.should.have.property('email', updateJSON.email);

        done();
      });
  });

  it('PUT Should return 500 when trying to Update User That Doesn\'t Exist', function(done){
    request
    .put('/casas/xxxxxxxxxxxxxxxx.json')
      .set('Accept', 'application/json')
      .expect('Content-Type', /json/)
      .send(updateJSON)
      .expect(500, done);
  });

  it('DELETE Should delete person when calling /casas/{ID}.json', function(done){
    request
      .del('/casas/'+newCasaId+'.json')
      .set('Accept', 'application/json')
      .expect('Content-Type', /json/)
      .expect(200, done);
  });

  it('DELETE Should return a 500 when calling /casas/{ID}.json with an ID that doesn\'t exist', function(done){
    request
      .del('/casas/xxxxxxxxxxxxxxxx.json')
      .set('Accept', 'application/json')
      .expect('Content-Type', /json/)
      .expect(500, done);
  });
})